-- VHDL Model Created from ECS Symbol tlc_engine.sym 
-- Jan 15 14:50:10 1997

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;
USE ieee.std_logic_unsigned.ALL;

---***** Clocing and Addressing for TI TLC5958 Led driver ****
entity tlc_engine is

      Port (     clk : In    std_logic;
                 run : In    std_logic;
	     addrclk : Out   std_logic;
	       gsclk : Out   std_logic;
	         eof : Out   std_logic;
             addrbus : Out   std_logic_vector (4 downto 0));

end tlc_engine;
	
architecture BEHAVIORAL of tlc_engine is

	-- **** On board CLK on CPLD pin 128 is used by default. @5MHz. ****
	-- **** Change interval constant accordingly for different frequency ****

	--Input GCLK continuously, 257GCLK (or 513GCLK) as a segment.
      	constant gsclk257 : Integer := 257;				-- Can be reduced here for analysing.

	-- Pulses counter.
      	signal j : integer range 0 to gsclk257 := 0;
 	
	--Between the interval of two segments,
	--LED supply voltage should be switched from one line to next line accordingly.
	--Interval between segments. This time depends on the discharge circuit of scan lines.
      	--It can be as short as possible, as long as no ghost phenomena observed.
      	--Suggested range is 0.5S~1S **** interval = 4 * (1/5MHz) = 4 * 0.2us = 0.8uS ****
      	constant interval : Integer := 4;		

      	--5 bits Addressing Bus Register.
      	signal ADDR : std_logic_vector(4 downto 0) := "00000";

	--Engines States
      	type state_typ is (idle, pulsing, addressing);
      	signal next_state : state_typ;

begin

	-- permanent behaviour.
	gsclk <= clk when (next_state = pulsing) else '0';
   	addrbus <= ADDR; -- type conversion 	
				
   process(clk, run,j)   
      begin
 
	--Since TLC5958 needs some time to prepare GS data for the 1st line of a new frame, after sending Vsync command (falling edge of LAT pulse).
	--Some wait time (2.5S) is needed before sending the 1st GCLK. 
	--**** Remember! CPU must drive low "run" input of CPLD for 2.5uS after falling edge of LAT pulse.****
      	 if (run = '0') then 
		-- Reset all
		next_state <= idle;
		j <= (gsclk257 -1);
   		ADDR <= "00000";
		eof <= '0';	
		addrclk <= '1';
	 elsif (falling_edge(clk)) then    				-- falling edge clock
		case next_state is
			when idle =>					-- GSCLK starts on next clock falling edge
				next_state <= pulsing;
				addrclk <= '1';
			when pulsing =>					-- 257 Pulses of GSCLK
				if (j = 0) then
					j <= (interval - 1);
					addrclk <= '0';
					if (ADDR = "11111") then
						ADDR <= "00000";
						eof <= '1';
					else
						ADDR <= ADDR + 1;
					end if;
					next_state <= addressing;	
				else
					j <= j-1;
				end if;
			when addressing =>				-- interval segment for addressing
				if (j = 0) then					
					j <= (gsclk257 - 1);			
					addrclk <= '1';
					eof <= '0';
					next_state <= pulsing;			
				else
					j <= j-1;			
				end if;
		end case;	
	 end if;

   end process;

end BEHAVIORAL;

